#version 330 core

// Interpolated values from the vertex shaders
//in vec3 fragmentColor;

in vec2 fragmentUV;
//in vec3 normal_cs;
//in vec3 lightdirection_cs[5];
in vec3 lightdirection_ts[5];
in vec3 position_ws;
in vec3 eyedirection_cs;
in vec3 eyedirection_ts;
in vec4 shadowCoord;
//in float aboveLight[5];
in mat3 tbn;

out vec4 fragColor;

uniform sampler2D colorTexture;
uniform sampler2D normalTexture;
uniform sampler2DShadow shadowMap;
uniform mat4 modelMatrix;
uniform mat4 viewMatrix;
uniform vec3 lightposition_ws[5];

vec2 poissonDisk[4] = vec2[](
  vec2( -0.94201624, -0.39906216 ),
  vec2( 0.94558609, -0.76890725 ),
  vec2( -0.094184101, -0.92938870 ),
  vec2( 0.34495938, 0.29387760 ) );

void main(void){

	vec4 textureColor = texture(colorTexture, fragmentUV);
	vec4 materialColor = textureColor; //vec4(textureColor, 1.0);
	vec4 ambient = vec4(0.1, 0.1, 0.1, 1) * materialColor;
	vec4 specular = vec4(0.3, 0.3, 0.3, 1);
	vec4 lightColor = vec4(1.0, 1.0, 1.0, 0.0);
	float lightPower = 100.0f;

	vec3 normal_ts = normalize(texture(normalTexture, vec2(fragmentUV.x, fragmentUV.y)).rgb*2.0 - 1.0);

	fragColor = ambient;

	for (int u = 0; u < lightposition_ws.length(); ++u) {
		//if (aboveLight[u] > -0.25) {
		//	fragColor *= 1.3;
		//	continue;
		//}
		
		// Distance to the light
		float distance = length(lightposition_ws[u] - position_ws);
		//if (distance > 15) {
		//	fragColor *= 1.1;
		//	continue;
		//}

		// lightning
		vec3 n = normal_ts;
		vec3 l = normalize(lightdirection_ts[u]);
		float cosTheta = clamp(dot(n, l), 0, 1);
		
		// speculars
		vec3 e = normalize(eyedirection_ts);
		vec3 r = reflect(-l,n);
		float cosAlpha = clamp(dot(e, r), 0, 1);

		// adapt to current light & shadowcoords etc
		float visibility = 1;
		float bias = clamp(0.005*tan(acos(cosTheta)), 0, 0.01);
		if (u == 1) {
			vec3 shadowMapCoord = shadowCoord.xyz/shadowCoord.w;
			for (int s=0;s<4;s++) {
				float depth = textureProj(shadowMap, vec4(shadowMapCoord.xyz+vec3(poissonDisk[s]/700.0,0),1));
				if (depth < (shadowMapCoord.z-bias)/shadowCoord.w) {
					visibility -= 0.25;
				}
				//visibility = (depth < (shadowMapCoord.z-bias)/shadowCoord.w) ? 0.0 : 1.0;
			}
			//visibility = texture(shadowMap, vec3(shadowCoord.xy, (shadowCoord.z)/shadowCoord.w));
		} 

		fragColor += visibility * materialColor * lightColor * lightPower * cosTheta / (distance * distance) + // used to be (distance * distance) but I didn't like it for testing purposes
					visibility * specular * lightColor * lightPower * pow(cosAlpha, 5) / (distance * distance); // used to be (distance * distance) but I didn't like it for testing purposes
	}	
	//fragColor.xyz = normalize(lightdirection_ts[1].xyz);		
}