#version 330 core

layout(location = 0) in vec3 position;
layout(location = 1) in vec2 uv;
layout(location = 2) in vec3 normal;
layout(location = 3) in vec3 tangent;
layout(location = 4) in vec3 bitangent;

//out vec3 normal_cs;
out vec2 fragmentUV;
out vec3 position_ws;
out vec3 eyedirection_cs;
out vec3 eyedirection_ts;
//out vec3 lightdirection_cs[5];
out vec3 lightdirection_ts[5];
out vec4 shadowCoord;
//out float[5] aboveLight;
out mat3 tbn;

// values from the main.cpp and/or the scene objects
uniform mat4 projectionMatrix;
uniform mat4 viewMatrix;
uniform mat4 modelMatrix;
uniform vec3 lightposition_ws[5];
uniform mat4 depthBiasVP;

void main() {
	
	mat4 mv = viewMatrix * modelMatrix;
	shadowCoord = depthBiasVP * modelMatrix * vec4(position, 1);
	fragmentUV = uv;
	gl_Position = projectionMatrix * mv * vec4(position, 1);
	// Position of the vertex, in worldspace : M * position
	position_ws = (modelMatrix * vec4(position,1)).xyz;
	
	// Vector that goes from the vertex to the camera, in camera space.
	// In camera space, the camera is at the origin (0,0,0).
	vec3 vertexposition_cs = (viewMatrix * modelMatrix * vec4(position,1)).xyz;
	vec3 eyedirection_cs = vec3(0,0,0) - vertexposition_cs;

	vec3 tangent_cs = mat3(mv) * tangent;
	vec3 bitangent_cs = mat3(mv) * bitangent;
	vec3 normal_cs = mat3(mv) * normal;
	//vec3 bitangent_cs = cross(normal_cs, tangent_cs);
	//tbn = mat3(	tangent_cs.x, bitangent_cs.x, normal_cs.x, 
	//			tangent_cs.y, bitangent_cs.y, normal_cs.y, 
	//			tangent_cs.z, bitangent_cs.z, normal_cs.z);
	tbn = transpose(mat3(tangent_cs, bitangent_cs, normal_cs));

	for (int u = 0; u < lightposition_ws.length(); ++u) {
		// Vector that goes from the vertex to the light, in camera space.	
		vec3 lightposition_cs = ( viewMatrix * vec4(lightposition_ws[u], 1)).xyz;
		vec3 lightdirection_cs = lightposition_cs + eyedirection_cs;
		lightdirection_ts[u] = tbn * lightdirection_cs;
		//aboveLight[u] = position_ws.y - lightposition_ws[u].y;
	}

	// 
	// Normal of the the vertex, in camera space
	//normal_cs = ( viewMatrix * modelMatrix * vec4(normal, 0)).xyz; // Only correct if ModelMatrix does not scale the model ! Use its inverse transpose if not.
	eyedirection_ts = tbn * eyedirection_cs;
}